//+--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|                                                                                                                                                             SymbolInstance.mqh |
//|                                                                                                                                                                            hoz |
//|                                                                                                                                                                                |
//+--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
#property copyright "hoz"
#property link      ""
#property version   "1.00"

//+================================================================================================================================================================================+
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//|                                                         КЛАСС ДЛЯ УПРОЩЁННОГО ДОСТУПА С СВОЙСТВАМ ТОРГУЕМОГО СИМВОЛА                                                           |
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+================================================================================================================================================================================+
class SymbolInstance {
  private:
    string   m_name;
    ENUM_ORDER_TYPE_FILLING   m_orderFillingType;    // Тип ордера по исполнению (указанным лотом в полном объёме, частичное исполнение и закрытие позиции с оставшимся объёмом или частичное исполнение позиции и продолжение действия с оставшимся меньшим объёмом)
  public:
    //---- Конструктор и деструктор
    SymbolInstance(const string symbolName);
   ~SymbolInstance(void);
    //---- Свойства
    ENUM_ORDER_TYPE_FILLING   getOrderFillingType(void) const;
    string   getName(void) const;
    MqlTick  getLastTick(void) const;
    bool     isSelected(void) const;
    bool     isVisible(void) const;
    long     lastQuoteTime(void) const;    // Возвращает время последней котировки
    int      digitsCount(void) const;
    double   spread(void) const;
    bool     isSpreadFloat(void) const;
    double   stopLevel(void) const;
    double   freezeLevel(void) const;
    ENUM_SYMBOL_TRADE_EXECUTION   conclusionContractMode(void) const;
    ENUM_SYMBOL_SWAP_MODE         swapCalculationModel(void) const;
    ENUM_DAY_OF_WEEK              swap3ChargeDay(void) const;
    bool     isMarketOrderTypeAllowed(void) const;
    bool     isLimitOrderTypeAllowed(void) const;
    bool     isStopOrderTypeAllowed(void) const;
    bool     isStopLimitOrderTypeAllowed(void) const;
    bool     isSLAllowed(void) const;
    bool     isTPAllowed(void) const;
    bool     isCloseByOrderTypeAllowed(void) const;
    double   pt(void) const;
    double   tickSize(void) const;
    double   commercialContractSize(void) const;   
    double   lotMin(void) const;
    double   lotMax(void) const;
    double   lotStep(void) const;
    double   lotLimit(void) const;
    double   swapLongValue(void) const;
    double   swapShortValue(void) const;
    double   marginInitial(void) const;
    double   marginMaintenance(void) const;
    double   sessionOpenPrice(void) const;
    double   sessionClosePrice(void) const;
    double   weightedAveragePrice(void) const;
    //---- Методы
    bool     checkForSelected(const bool select) const;
    void     setOrderFillingType(void);
};
//==================================================================================================================================================================================
// Конструктор класса. =============================================================================================================================================================
SymbolInstance :: SymbolInstance(const string symbolName) {
  m_name = symbolName;
  setOrderFillingType();
}
//==================================================================================================================================================================================
// Деструктор класса. ==============================================================================================================================================================
SymbolInstance :: ~SymbolInstance(void) {}
//==================================================================================================================================================================================
// Возвращает тип ордера по исполнению(m_orderFillingType). ========================================================================================================================
ENUM_ORDER_TYPE_FILLING SymbolInstance :: getOrderFillingType(void) const {
//---
  return m_orderFillingType;
}
//==================================================================================================================================================================================
// Возвращает символ инструмента. ==================================================================================================================================================
string SymbolInstance :: getName(void) const {
//---
  return m_name;
}
//==================================================================================================================================================================================
// Возвращает данные последнего тика. ==============================================================================================================================================
MqlTick SymbolInstance :: getLastTick(void) const {
//---
  MqlTick lastTick;
  SymbolInfoTick(m_name, lastTick);
  return lastTick;
}
//==================================================================================================================================================================================
// Возвращает признак того, что символ выбран в окне MarketWatch(обзор рынка). =====================================================================================================
bool SymbolInstance :: isSelected(void) const {
//---
  return (bool)SymbolInfoInteger(m_name, SYMBOL_SELECT);
}
//==================================================================================================================================================================================
// Возвращает признак того, что символ отображается в окне MarketWatch(обзор рынка). ===============================================================================================
bool SymbolInstance :: isVisible(void) const {
//---
  return (bool)SymbolInfoInteger(m_name, SYMBOL_VISIBLE);
}
//==================================================================================================================================================================================
// Возвращает время последней котировки. ===========================================================================================================================================
long SymbolInstance :: lastQuoteTime(void) const {
//---
  return SymbolInfoInteger(m_name, SYMBOL_TIME);
}
//==================================================================================================================================================================================
// Возвращает количество знаков после запятой, в цене торгового инструмента. =======================================================================================================
int SymbolInstance :: digitsCount(void) const {
//---
  return (int)SymbolInfoInteger(m_name, SYMBOL_DIGITS);
}
//==================================================================================================================================================================================
// Возвращает текущее значение спреда, в пунктах. ==================================================================================================================================
double SymbolInstance :: spread(void) const {
//---
  return SymbolInfoInteger(m_name, SYMBOL_SPREAD) * SymbolInfoDouble(m_name, SYMBOL_POINT);
}
//==================================================================================================================================================================================
// Возвращает признак того, что спред является плавающий. ==========================================================================================================================
bool SymbolInstance :: isSpreadFloat(void) const {
//---
  return (bool)SymbolInfoInteger(m_name, SYMBOL_SPREAD_FLOAT);
}
//==================================================================================================================================================================================
// Возвращает минимальный отступ от текущей цены до ближайшей цены, где возможна установка Stop-ордеров (SL или TP), в пунктах. ====================================================
double SymbolInstance :: stopLevel(void) const {
//---
  return SymbolInfoInteger(m_name, SYMBOL_TRADE_STOPS_LEVEL) * SymbolInfoDouble(m_name, SYMBOL_POINT);
}
//==================================================================================================================================================================================
// Возвращает значение дистенции заморозки торговых операций. ======================================================================================================================
double SymbolInstance :: freezeLevel(void) const {
//---
  return SymbolInfoInteger(m_name, SYMBOL_TRADE_FREEZE_LEVEL) * SymbolInfoDouble(m_name, SYMBOL_POINT);
}
//==================================================================================================================================================================================
// Возвращает режим заключения сделок по инструменту. ==============================================================================================================================
ENUM_SYMBOL_TRADE_EXECUTION SymbolInstance :: conclusionContractMode(void) const {
//---
  return (ENUM_SYMBOL_TRADE_EXECUTION)SymbolInfoInteger(m_name, SYMBOL_TRADE_EXEMODE);
}
//==================================================================================================================================================================================
// Возвращает модель расчёта свопа. ================================================================================================================================================
ENUM_SYMBOL_SWAP_MODE SymbolInstance :: swapCalculationModel(void) const {
//---
  return (ENUM_SYMBOL_SWAP_MODE)SymbolInfoInteger(m_name, SYMBOL_SWAP_MODE);
}
//==================================================================================================================================================================================
// Возвращает день недели начисления тройного свопа. ===============================================================================================================================
ENUM_DAY_OF_WEEK SymbolInstance :: swap3ChargeDay(void) const {
//---
  return (ENUM_DAY_OF_WEEK)SymbolInfoInteger(m_name, SYMBOL_SWAP_ROLLOVER3DAYS);
}
//==================================================================================================================================================================================
// Разрешены ли рыночные ордера(Buy и Sell). =======================================================================================================================================
bool SymbolInstance :: isMarketOrderTypeAllowed(void) const {
// получим значение свойства, описывающего разрешенные типы ордеров
  int allowedOrderTypes = (int)SymbolInfoInteger(m_name, SYMBOL_ORDER_MODE);
  return ((SYMBOL_ORDER_MARKET& allowedOrderTypes) == SYMBOL_ORDER_MARKET) ? true : false;
}
//==================================================================================================================================================================================
// Разрешены ли лимитные ордера(Buy Limit и Sell Limit). ===========================================================================================================================
bool SymbolInstance :: isLimitOrderTypeAllowed(void) const {
// получим значение свойства, описывающего разрешенные типы ордеров
  int allowedOrderTypes = (int)SymbolInfoInteger(m_name, SYMBOL_ORDER_MODE);
  return ((SYMBOL_ORDER_LIMIT& allowedOrderTypes) == SYMBOL_ORDER_LIMIT) ? true : false;
}
//==================================================================================================================================================================================
// Разрешены ли стоп-ордера(Buy Stop и Sell Stop). =================================================================================================================================
bool SymbolInstance :: isStopOrderTypeAllowed(void) const {
// получим значение свойства, описывающего разрешенные типы ордеров
  int allowedOrderTypes = (int)SymbolInfoInteger(m_name, SYMBOL_ORDER_MODE);
  return ((SYMBOL_ORDER_STOP& allowedOrderTypes) == SYMBOL_ORDER_STOP) ? true : false;
}
//==================================================================================================================================================================================
// Разрешены ли стоп-лимит ордера(Buy Stop Limit и Sell Stop Limit). ===============================================================================================================
bool SymbolInstance :: isStopLimitOrderTypeAllowed(void) const {
// получим значение свойства, описывающего разрешенные типы ордеров
  int allowedOrderTypes = (int)SymbolInfoInteger(m_name, SYMBOL_ORDER_MODE);
  return ((SYMBOL_ORDER_STOP_LIMIT& allowedOrderTypes) == SYMBOL_ORDER_STOP_LIMIT) ? true : false;
}
//==================================================================================================================================================================================
// Разрешена ли установка SL(Stop Loss). ===========================================================================================================================================
bool SymbolInstance :: isSLAllowed(void) const {
// получим значение свойства, описывающего разрешенные типы ордеров
  int allowedOrderTypes = (int)SymbolInfoInteger(m_name, SYMBOL_ORDER_MODE);
  return ((SYMBOL_ORDER_SL& allowedOrderTypes) == SYMBOL_ORDER_SL) ? true : false;
}
//==================================================================================================================================================================================
// Разрешена ли установка TP(Take Profit). =========================================================================================================================================
bool SymbolInstance :: isTPAllowed(void) const {
// получим значение свойства, описывающего разрешенные типы ордеров
  int allowedOrderTypes = (int)SymbolInfoInteger(m_name, SYMBOL_ORDER_MODE);
  return ((SYMBOL_ORDER_TP& allowedOrderTypes) == SYMBOL_ORDER_TP) ? true : false;
}
//==================================================================================================================================================================================
// Разрешено ли ли закрытие позиций встречной. =====================================================================================================================================
bool SymbolInstance :: isCloseByOrderTypeAllowed(void) const {
// получим значение свойства, описывающего разрешенные типы ордеров
  int allowedOrderTypes = (int)SymbolInfoInteger(m_name, SYMBOL_ORDER_MODE);
  return ((SYMBOL_ORDER_CLOSEBY& allowedOrderTypes) == SYMBOL_ORDER_CLOSEBY) ? true : false;
}
//==================================================================================================================================================================================
// Выбирает переданный символ в окне MarketWatch(обзор рынка) или убирает его из этого окна. =======================================================================================
bool SymbolInstance :: checkForSelected(const bool select) const {    // Флаг. Если значение false, то символ должен быть убран из окна MarketWatch, иначе символ должен быть выбран в окне MarketWatch.
//---
  return SymbolSelect(m_name, select);
}
//==================================================================================================================================================================================
// Setter для политики исполнения позиции(m_orderFillingType). =====================================================================================================================
void SymbolInstance :: setOrderFillingType(void) {
//---
  int fillingMode = (int)SymbolInfoInteger(m_name, SYMBOL_FILLING_MODE);    // Получение возможного варианта заполнения ордера

  if ((SYMBOL_FILLING_FOK& fillingMode) == SYMBOL_FILLING_FOK) {
    m_orderFillingType = ORDER_FILLING_FOK;
  } else {
    if ((SYMBOL_FILLING_IOC& fillingMode) == SYMBOL_FILLING_IOC) {
      m_orderFillingType = ORDER_FILLING_IOC;
    } else {
      m_orderFillingType = ORDER_FILLING_RETURN;
    }
  }
}
//==================================================================================================================================================================================
// Возвращает размер пункта. =======================================================================================================================================================
double SymbolInstance :: pt(void) const {
//---
  return SymbolInfoDouble(m_name, SYMBOL_POINT);
}
//==================================================================================================================================================================================
// Djpdhfoftn размер тика. =========================================================================================================================================================
double SymbolInstance :: tickSize(void) const {
//---
  return SymbolInfoDouble(m_name, SYMBOL_TRADE_TICK_SIZE);
}
//==================================================================================================================================================================================
// Возвращает размер торгового контракта. ==========================================================================================================================================
double SymbolInstance :: commercialContractSize(void) const {
//---
  return SymbolInfoDouble(m_name, SYMBOL_TRADE_CONTRACT_SIZE);
}
//==================================================================================================================================================================================
// Возвращает минимальный лот для заключения сделки. ===============================================================================================================================
double SymbolInstance :: lotMin(void) const {
//---
  return SymbolInfoDouble(m_name, SYMBOL_VOLUME_MIN);
}
//==================================================================================================================================================================================
// Возвращает максимальный лот для заключения сделки. ==============================================================================================================================
double SymbolInstance :: lotMax(void) const {
//---
  return SymbolInfoDouble(m_name, SYMBOL_VOLUME_MAX);
}
//==================================================================================================================================================================================
// Возвращает минимальный шаг изменения размера лота для заключения сделки. ========================================================================================================
double SymbolInstance :: lotStep(void) const {
//---
  return SymbolInfoDouble(m_name, SYMBOL_VOLUME_STEP);
}
//==================================================================================================================================================================================
// Возвращает максимально допустимый для данного символа совокупный объем открытой позиции и отложенных ордеров  в одном направлении (покупка или продажа). ========================
double SymbolInstance :: lotLimit(void) const {
//---
  return SymbolInfoDouble(m_name, SYMBOL_VOLUME_LIMIT);
}
//==================================================================================================================================================================================
// Возвращает значение свопа на long-позицию. ======================================================================================================================================
double SymbolInstance :: swapLongValue(void) const {
//---
  return SymbolInfoDouble(m_name, SYMBOL_SWAP_LONG);
}
//==================================================================================================================================================================================
// Возвращает значение свопа на short-позицию. =====================================================================================================================================
double SymbolInstance :: swapShortValue(void) const {
//---
  return SymbolInfoDouble(m_name, SYMBOL_SWAP_SHORT);
}
//==================================================================================================================================================================================
// Возвращает размер начальной маржи(залоговых средств в маржинальной валюте инструмента) необходимых для открытия позиции объёмом в один лот. Используется при проверке средств клиента при входе в рынок.
double SymbolInstance :: marginInitial(void) const {
//---
  return SymbolInfoDouble(m_name, SYMBOL_MARGIN_INITIAL);
}
//==================================================================================================================================================================================
// Возвращает размер поддерживающей маржи(залоговых средств в маржинальной валюте инструмента), удерживаемую с одного лота. Используется при проверке средств клиента при изменении
// состояния счета клиента. Если поддерживающая маржа равна 0, то используется начальная маржа. ====================================================================================
double SymbolInstance :: marginMaintenance(void) const {
//---
  return SymbolInfoDouble(m_name, SYMBOL_MARGIN_MAINTENANCE);
}
//==================================================================================================================================================================================
// Возвращает цену открытия сессии. ================================================================================================================================================
double SymbolInstance :: sessionOpenPrice(void) const {
//---
  return SymbolInfoDouble(m_name, SYMBOL_SESSION_OPEN);
}
//==================================================================================================================================================================================
// Возвращает цену закрытия сессии. ================================================================================================================================================
double SymbolInstance :: sessionClosePrice(void) const {
//---
  return SymbolInfoDouble(m_name, SYMBOL_SESSION_CLOSE);
}
//==================================================================================================================================================================================
// Возвращает средневзвешенную цену сессии. ========================================================================================================================================
double SymbolInstance :: weightedAveragePrice(void) const {
//---
  return SymbolInfoDouble(m_name, SYMBOL_SESSION_AW);
}